@@ -75,25 +75,25 @@ iDeclareType(App)
 
 #if defined (iPlatformApple)
 #define EMB_BIN "../../Resources/resources.lgr"
-static const char *dataDir_App_ = "~/Library/Application Support/fi.skyjake.Lagrange";
+static const char *defaultDataDir_App_ = "~/Library/Application Support/fi.skyjake.Lagrange";
 #endif
 #if defined (iPlatformMsys)
 #define EMB_BIN "../resources.lgr"
-static const char *dataDir_App_ = "~/AppData/Roaming/fi.skyjake.Lagrange";
+static const char *defaultDataDir_App_ = "~/AppData/Roaming/fi.skyjake.Lagrange";
 #endif
 #if defined (iPlatformLinux) || defined (iPlatformOther)
 #define EMB_BIN  "../../share/lagrange/resources.lgr"
-static const char *dataDir_App_ = "~/.config/lagrange";
+static const char *defaultDataDir_App_ = "~/.config/lagrange";
 #endif
 #if defined (LAGRANGE_EMB_BIN) /* specified in build config */
 #  undef EMB_BIN
 #  define EMB_BIN LAGRANGE_EMB_BIN
 #endif
 #define EMB_BIN2 "../resources.lgr" /* fallback from build/executable dir */
-static const char *prefsFileName_App_    = "prefs.cfg";
-static const char *oldStateFileName_App_ = "state.binary";
-static const char *stateFileName_App_    = "state.lgr";
-static const char *downloadDir_App_      = "~/Downloads";
+static const char *prefsFileName_App_      = "prefs.cfg";
+static const char *oldStateFileName_App_   = "state.binary";
+static const char *stateFileName_App_      = "state.lgr";
+static const char *defaultDownloadDir_App_ = "~/Downloads";
 
 static const int idleThreshold_App_ = 1000; /* ms */
 
@@ -213,14 +213,42 @@ static iString *serializePrefs_App_(const iApp *d) {
     return str;
 }
 
+static const char *dataDir_App_(void) {
+#if defined (iPlatformLinux) || defined (iPlatformOther)
+    const char *configHome = getenv("XDG_CONFIG_HOME");
+    if (configHome) {
+        return concatPath_CStr(configHome, "lagrange");
+    }
+#endif
+    return defaultDataDir_App_;
+}
+
+static const char *downloadDir_App_(void) {
+#if defined (iPlatformLinux) || defined (iPlatformOther)
+    /* Parse user-dirs.dirs using the `xdg-user-dir` tool. */
+    iProcess *proc = iClob(new_Process());
+    setArguments_Process(
+        proc, iClob(newStringsCStr_StringList("/usr/bin/env", "xdg-user-dir", "DOWNLOAD", NULL)));
+    if (start_Process(proc)) {
+        iString *path = collect_String(newLocal_String(collect_Block(
+            readOutputUntilClosed_Process(proc))));
+        trim_String(path);
+        if (!isEmpty_String(path)) {
+            return cstr_String(path);
+        }
+    }
+#endif
+    return defaultDownloadDir_App_;
+}
+
 static const iString *prefsFileName_(void) {
-    return collect_String(concatCStr_Path(&iStringLiteral(dataDir_App_), prefsFileName_App_));
+    return collectNewCStr_String(concatPath_CStr(dataDir_App_(), prefsFileName_App_));
 }
 
 static void loadPrefs_App_(iApp *d) {
     iUnused(d);
     /* Create the data dir if it doesn't exist yet. */
-    makeDirs_Path(collectNewCStr_String(dataDir_App_));
+    makeDirs_Path(collectNewCStr_String(dataDir_App_()));
     iFile *f = new_File(prefsFileName_());
     if (open_File(f, readOnly_FileMode | text_FileMode)) {
         iString *str = readString_File(f);
@@ -267,8 +295,8 @@ static const char *magicTabDocument_App_ = "tabd";
 
 static iBool loadState_App_(iApp *d) {
     iUnused(d);
-    const char *oldPath = concatPath_CStr(dataDir_App_, oldStateFileName_App_);
-    const char *path    = concatPath_CStr(dataDir_App_, stateFileName_App_);
+    const char *oldPath = concatPath_CStr(dataDir_App_(), oldStateFileName_App_);
+    const char *path    = concatPath_CStr(dataDir_App_(), stateFileName_App_);
     iFile *f = iClob(newCStr_File(fileExistsCStr_FileInfo(path) ? path : oldPath));
     if (open_File(f, readOnly_FileMode)) {
         char magic[4];
@@ -323,7 +351,7 @@ iObjectList *listDocuments_App(void) {
 static void saveState_App_(const iApp *d) {
     iUnused(d);
     trimCache_App();
-    iFile *f = newCStr_File(concatPath_CStr(dataDir_App_, stateFileName_App_));
+    iFile *f = newCStr_File(concatPath_CStr(dataDir_App_(), stateFileName_App_));
     if (open_File(f, writeOnly_FileMode)) {
         writeData_File(f, magicState_App_, 4);
         writeU32_File(f, latest_FileVersion); /* version */
@@ -340,6 +368,7 @@ static void saveState_App_(const iApp *d) {
 #if defined (LAGRANGE_IDLE_SLEEP)
 static uint32_t checkAsleep_App_(uint32_t interval, void *param) {
     iApp *d = param;
+    iUnused(d);
     SDL_Event ev = { .type = SDL_USEREVENT };
     ev.user.code = asleep_UserEventCode;
     SDL_PushEvent(&ev);
@@ -348,12 +377,14 @@ static uint32_t checkAsleep_App_(uint32_t interval, void *param) {
 #endif
 
 static void init_App_(iApp *d, int argc, char **argv) {
-    const iBool isFirstRun = !fileExistsCStr_FileInfo(cleanedPath_CStr(dataDir_App_));
+    const iBool isFirstRun = !fileExistsCStr_FileInfo(cleanedPath_CStr(dataDir_App_()));
     d->isFinishedLaunching = iFalse;
     d->launchCommands      = new_StringList();
     iZap(d->lastDropTime);
     init_CommandLine(&d->args, argc, argv);
-    /* Where was the app started from? */ {
+    /* Where was the app started from? We ask SDL first because the command line alone is
+       not a reliable source of this information, particularly when it comes to different
+       operating systems. */ {
         char *exec = SDL_GetBasePath();
         if (exec) {
             d->execPath = newCStr_String(concatPath_CStr(
@@ -373,14 +404,22 @@ static void init_App_(iApp *d, int argc, char **argv) {
 #if defined (iPlatformMsys)
     /* Must scale by UI scaling factor. */
     mulfv_I2(&d->initialWindowRect.size, desktopDPI_Win32());
+#endif
+#if defined (iPlatformLinux)
+    /* Scale by the primary (?) monitor DPI. */ {
+        float vdpi;
+        SDL_GetDisplayDPI(0, NULL, NULL, &vdpi);
+        const float factor = vdpi / 96.0f;
+        mulfv_I2(&d->initialWindowRect.size, iMax(factor, 1.0f));
+    }
 #endif
     init_Prefs(&d->prefs);
-    setCStr_String(&d->prefs.downloadDir, downloadDir_App_);
+    setCStr_String(&d->prefs.downloadDir, downloadDir_App_());
     d->isRunning         = iFalse;
     d->window            = NULL;
     set_Atomic(&d->pendingRefresh, iFalse);
     d->mimehooks         = new_MimeHooks();
-    d->certs             = new_GmCerts(dataDir_App_);
+    d->certs             = new_GmCerts(dataDir_App_());
     d->visited           = new_Visited();
     d->bookmarks         = new_Bookmarks();
     d->tabEnum           = 0; /* generates unique IDs for tab pages */
@@ -395,32 +434,23 @@ static void init_App_(iApp *d, int argc, char **argv) {
 #endif
     init_Keys();
     loadPrefs_App_(d);
-    load_Keys(dataDir_App_);
-    load_Visited(d->visited, dataDir_App_);
-    load_Bookmarks(d->bookmarks, dataDir_App_);
-    load_MimeHooks(d->mimehooks, dataDir_App_);
+    load_Keys(dataDir_App_());
+    load_Visited(d->visited, dataDir_App_());
+    load_Bookmarks(d->bookmarks, dataDir_App_());
+    load_MimeHooks(d->mimehooks, dataDir_App_());
     if (isFirstRun) {
         /* Create the default bookmarks for a quick start. */
-        add_Bookmarks(d->bookmarks,
-                      collectNewCStr_String("gemini://gemini.circumlunar.space/"),
-                      collectNewCStr_String("Project Gemini"),
-                      NULL,
-                      0x264a /* Gemini symbol */);
-        add_Bookmarks(d->bookmarks,
-                      collectNewCStr_String("gemini://gemini.circumlunar.space/capcom/"),
-                      collectNewCStr_String("CAPCOM Geminispace aggregator"),
-                      NULL,
-                      0x264a /* Gemini symbol */);
-        add_Bookmarks(d->bookmarks,
-                      collectNewCStr_String("gemini://gus.guru/"),
-                      collectNewCStr_String("GUS - Gemini Universal Search"),
-                      NULL,
-                      0x2690);
         add_Bookmarks(d->bookmarks,
                       collectNewCStr_String("gemini://skyjake.fi/lagrange/"),
                       collectNewCStr_String("Lagrange"),
                       NULL,
                       0x1f306);
+        add_Bookmarks(d->bookmarks,
+                      collectNewCStr_String("gemini://skyjake.fi/lagrange/getting_started.gmi"),
+                      collectNewCStr_String("Getting Started"),
+                      collectNewCStr_String("remotesource"),
+                      0x1f306);
+        fetchRemote_Bookmarks(d->bookmarks);
     }
 #if defined (iHaveLoadEmbed)
     /* Load the resources from a file. */ {
@@ -433,7 +463,7 @@ static void init_App_(iApp *d, int argc, char **argv) {
     }
 #endif
     d->window = new_Window(d->initialWindowRect);
-    init_Feeds(dataDir_App_);
+    init_Feeds(dataDir_App_());
     /* Widget state init. */
     processEvents_App(postedEventsOnly_AppEventMode);
     if (!loadState_App_(d)) {
@@ -468,13 +498,13 @@ static void init_App_(iApp *d, int argc, char **argv) {
 static void deinit_App(iApp *d) {
     saveState_App_(d);
     deinit_Feeds();
-    save_Keys(dataDir_App_);
+    save_Keys(dataDir_App_());
     deinit_Keys();
     savePrefs_App_(d);
     deinit_Prefs(&d->prefs);
-    save_Bookmarks(d->bookmarks, dataDir_App_);
+    save_Bookmarks(d->bookmarks, dataDir_App_());
     delete_Bookmarks(d->bookmarks);
-    save_Visited(d->visited, dataDir_App_);
+    save_Visited(d->visited, dataDir_App_());
     delete_Visited(d->visited);
     delete_GmCerts(d->certs);
     save_MimeHooks(d->mimehooks);
@@ -493,7 +523,7 @@ const iString *execPath_App(void) {
 }
 
 const iString *dataDir_App(void) {
-    return collect_String(cleanedCStr_Path(dataDir_App_));
+    return collect_String(cleanedCStr_Path(dataDir_App_()));
 }
 
 const iString *downloadDir_App(void) {
@@ -501,6 +531,7 @@ const iString *downloadDir_App(void) {
 }
 
 const iString *debugInfo_App(void) {
+    extern char **environ; /* The environment variables. */
     iApp *d = &app_;
     iString *msg = collectNew_String();
     format_String(msg, "# Debug information\n");
@@ -512,6 +543,11 @@ const iString *debugInfo_App(void) {
                             cstr_String(bookmarkTitle_DocumentWidget(doc)));
         append_String(msg, collect_String(debugInfo_History(history_DocumentWidget(doc))));
     }
+    appendCStr_String(msg, "## Environment\n```\n");
+    for (char **env = environ; *env; env++) {
+        appendFormat_String(msg, "%s\n", *env);
+    }
+    appendCStr_String(msg, "```\n");
     appendFormat_String(msg, "## Launch arguments\n```\n");
     iConstForEach(StringList, i, args_CommandLine(&d->args)) {
         appendFormat_String(msg, "%3zu : %s\n", i.pos, cstr_String(i.value));
@@ -519,7 +555,7 @@ const iString *debugInfo_App(void) {
     appendFormat_String(msg, "```\n## Launch commands\n");
     iConstForEach(StringList, j, d->launchCommands) {
         appendFormat_String(msg, "%s\n", cstr_String(j.value));
-    }    
+    }
     appendFormat_String(msg, "## MIME hooks\n");
     append_String(msg, debugInfo_MimeHooks(d->mimehooks));
     return msg;
@@ -653,7 +689,7 @@ static int resizeWatcher_(void *user, SDL_Event *event) {
             SDL_Event u = { .type = SDL_USEREVENT };
             u.user.code = command_UserEventCode;
             u.user.data1 = strdup("theme.changed");
-            u.user.windowID = SDL_GetWindowID(d->window->win);
+            u.user.windowID = id_Window(d->window);
             dispatchEvent_Widget(d->window->root, &u);
         }
 #endif
@@ -746,7 +782,7 @@ void postRefresh_App(void) {
         SDL_Event ev;
         ev.user.type     = SDL_USEREVENT;
         ev.user.code     = refresh_UserEventCode;
-        ev.user.windowID = get_Window() ? SDL_GetWindowID(get_Window()->win) : 0;
+        ev.user.windowID = id_Window(get_Window());
         ev.user.data1    = NULL;
         ev.user.data2    = NULL;
         SDL_PushEvent(&ev);
@@ -771,7 +807,7 @@ void postCommand_App(const char *command) {
     }
     ev.user.type     = SDL_USEREVENT;
     ev.user.code     = command_UserEventCode;
-    ev.user.windowID = get_Window() ? SDL_GetWindowID(get_Window()->win) : 0;
+    ev.user.windowID = id_Window(get_Window());
     ev.user.data1    = strdup(command);
     ev.user.data2    = NULL;
     SDL_PushEvent(&ev);
@@ -847,8 +883,10 @@ static iBool handlePrefsCommands_(iWidget *d, const char *cmd) {
     if (equal_Command(cmd, "prefs.dismiss") || equal_Command(cmd, "preferences")) {
         setUiScale_Window(get_Window(),
                           toFloat_String(text_InputWidget(findChild_Widget(d, "prefs.uiscale"))));
+#if defined (LAGRANGE_DOWNLOAD_EDIT)
         postCommandf_App("downloads path:%s",
                          cstr_String(text_InputWidget(findChild_Widget(d, "prefs.downloads"))));
+#endif
         postCommandf_App("window.retain arg:%d",
                          isSelected_Widget(findChild_Widget(d, "prefs.retainwindow")));
         postCommandf_App("smoothscroll arg:%d",
@@ -1002,7 +1040,7 @@ static iBool handleIdentityCreationCommands_(iWidget *dlg, const char *cmd) {
                     sscanf(cstr_String(text_InputWidget(findChild_Widget(dlg, "ident.until"))),
                            "%04u-%u-%u %u:%u:%u",
                            &val[0], &val[1], &val[2], &val[3], &val[4], &val[5]);
-                if (n <= 0 || val[0] < (unsigned) today.year) {
+                if (n <= 0) {
                     makeMessage_Widget(orange_ColorEscape "INVALID DATE",
                                        "Please check the \"Valid until\" date. Examples:\n"
                                        "\u2022 2030\n"
@@ -1016,6 +1054,7 @@ static iBool handleIdentityCreationCommands_(iWidget *dlg, const char *cmd) {
                 until.hour   = n >= 4 ? val[3] : 0;
                 until.minute = n >= 5 ? val[4] : 0;
                 until.second = n == 6 ? val[5] : 0;
+                until.gmtOffsetSeconds = today.gmtOffsetSeconds;
                 /* In the past? */ {
                     iTime now, t;
                     initCurrent_Time(&now);
@@ -1441,7 +1480,7 @@ iBool handleCommand_App(const char *cmd) {
         return iTrue;
     }
     else if (equal_Command(cmd, "bookmarks.changed")) {
-        save_Bookmarks(d->bookmarks, dataDir_App_);
+        save_Bookmarks(d->bookmarks, dataDir_App_());
         return iFalse;
     }
     else if (equal_Command(cmd, "feeds.refresh")) {
@@ -1460,11 +1499,12 @@ iBool handleCommand_App(const char *cmd) {
         return iFalse;
     }
     else if (equal_Command(cmd, "visited.changed")) {
-        save_Visited(d->visited, dataDir_App_);
+        save_Visited(d->visited, dataDir_App_());
         return iFalse;
     }
     else if (equal_Command(cmd, "ident.new")) {
         iWidget *dlg = makeIdentityCreation_Widget();
+        setFocus_Widget(findChild_Widget(dlg, "ident.until"));
         setCommandHandler_Widget(dlg, handleIdentityCreationCommands_);
         return iTrue;
     }
@@ -1538,7 +1578,7 @@ void openInDefaultBrowser_App(const iString *url) {
 
 void revealPath_App(const iString *path) {
 #if defined (iPlatformApple)
-    const char *scriptPath = concatPath_CStr(dataDir_App_, "revealfile.scpt");
+    const char *scriptPath = concatPath_CStr(dataDir_App_(), "revealfile.scpt");
     iFile *f = newCStr_File(scriptPath);
     if (open_File(f, writeOnly_FileMode | text_FileMode)) {
         /* AppleScript to select a specific file. */
