@@ -177,7 +177,6 @@ const iString *text_InputWidget(const iInputWidget *d) {
 void setMaxLen_InputWidget(iInputWidget *d, size_t maxLen) {
     d->maxLen = maxLen;
     d->mode   = (maxLen == 0 ? insert_InputMode : overwrite_InputMode);
-    resize_Array(&d->text, maxLen);
     if (maxLen) {
         /* Set a fixed size. */
         iBlock *content = new_Block(maxLen);
@@ -224,6 +223,7 @@ static void updateBuffered_InputWidget_(iInputWidget *d) {
 }
 
 void setText_InputWidget(iInputWidget *d, const iString *text) {
+    if (!d) return;
     if (d->inFlags & isUrl_InputWidgetFlag) {
         /* If user wants URLs encoded, also Punycode the domain. */
         if (!prefs_App()->decodeUserVisibleURLs) {
@@ -332,6 +332,7 @@ void end_InputWidget(iInputWidget *d, iBool accept) {
 }
 
 static void insertChar_InputWidget_(iInputWidget *d, iChar chr) {
+    iWidget *w = as_Widget(d);
     if (d->mode == insert_InputMode) {
         insert_Array(&d->text, d->cursor, &chr);
         d->cursor++;
@@ -342,7 +343,8 @@ static void insertChar_InputWidget_(iInputWidget *d, iChar chr) {
         }
         set_Array(&d->text, d->cursor++, &chr);
         if (d->maxLen && d->cursor == d->maxLen) {
-            setFocus_Widget(NULL);
+            iWidget *nextFocus = findFocusable_Widget(w, forward_WidgetFocusDir);
+            setFocus_Widget(nextFocus == w ? NULL : nextFocus);
         }
     }
     showCursor_InputWidget_(d);
