@@ -56,7 +56,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 
 static iWindow *theWindow_ = NULL;
 
-#if defined (iPlatformApple)
+#if defined (iPlatformApple) || defined (iPlatformLinux)
 static float initialUiScale_ = 1.0f;
 #else
 static float initialUiScale_ = 1.1f;
@@ -667,9 +667,16 @@ static void updateRootSize_Window_(iWindow *d, iBool notifyAlways) {
     const iInt2 oldSize = *size;
     SDL_GetRendererOutputSize(d->render, &size->x, &size->y);
     if (notifyAlways || !isEqual_I2(oldSize, *size)) {
+        const iBool isHoriz = (d->lastNotifiedSize.x != size->x);
+        const iBool isVert  = (d->lastNotifiedSize.y != size->y);
         arrange_Widget(d->root);
-        postCommandf_App("window.resized width:%d height:%d", size->x, size->y);
+        postCommandf_App("window.resized width:%d height:%d horiz:%d vert:%d",
+                         size->x,
+                         size->y,
+                         isHoriz,
+                         isVert);
         postRefresh_App();
+        d->lastNotifiedSize = *size;
     }
 }
 
@@ -685,7 +692,12 @@ static float pixelRatio_Window_(const iWindow *d) {
 #if defined (iPlatformMsys)
     iUnused(d);
     return desktopDPI_Win32();
-#else
+#elif defined (iPlatformLinux)
+    float vdpi = 0.0f;
+    SDL_GetDisplayDPI(SDL_GetWindowDisplayIndex(d->win), NULL, NULL, &vdpi);
+    const float factor = vdpi / 96.0f;
+    return iMax(1.0f, factor);
+#else    
     int dx, x;
     SDL_GetRendererOutputSize(d->render, &dx, NULL);
     SDL_GetWindowSize(d->win, &x, NULL);
@@ -714,6 +726,7 @@ void init_Window(iWindow *d, iRect rect) {
     iZap(d->cursors);
     d->initialPos = rect.pos;
     d->lastRect = rect;
+    d->lastNotifiedSize = zero_I2();
     d->pendingCursor = NULL;
     d->isDrawFrozen = iTrue;
     d->isMouseInside = iTrue;
@@ -764,6 +777,7 @@ void init_Window(iWindow *d, iRect rect) {
     useExecutableIconResource_SDLWindow(d->win);
 #endif
 #if defined (iPlatformLinux)
+    SDL_SetWindowMinimumSize(d->win, minSize.x * d->pixelRatio, minSize.y * d->pixelRatio);
     /* Load the window icon. */ {
         int w, h, num;
         const iBlock *icon = &imageLagrange64_Embedded;
@@ -781,6 +795,7 @@ void init_Window(iWindow *d, iRect rect) {
     }
 #endif
     d->root = new_Widget();
+    setFlags_Widget(d->root, focusRoot_WidgetFlag, iTrue);
     d->presentTime = 0.0;
     d->frameTime = SDL_GetTicks();
     d->loadAnimTimer = 0;
@@ -1027,12 +1042,16 @@ void setCursor_Window(iWindow *d, int cursor) {
     d->pendingCursor = d->cursors[cursor];
 }
 
+uint32_t id_Window(const iWindow *d) {
+    return d && d->win ? SDL_GetWindowID(d->win) : 0;
+}
+
 iInt2 rootSize_Window(const iWindow *d) {
     return d->root->rect.size;
 }
 
 iInt2 coord_Window(const iWindow *d, int x, int y) {
-#if defined (iPlatformMsys)
+#if defined (iPlatformMsys) || defined (iPlatformLinux)
     /* On Windows, surface coordinates are in pixels. */
     return init_I2(x, y);
 #else
